import { system, world } from '@minecraft/server';
import { spawnItem, spawnParticle, spawnEntity, isPickaxe, playSound} from './libs/utils.js';
import { ModalFormData } from "@minecraft/server-ui";



const loreLine = '§r §cAuto§eSmelt';

const smeltableOres = [
  "minecraft:copper_ore",
  "minecraft:deepslate_copper_ore",
  "minecraft:iron_ore",
  "minecraft:deepslate_iron_ore",
  "minecraft:gold_ore",
  "minecraft:deepslate_gold_ore",
  "minecraft:ancient_debris"
];

const smeltingBlocks = {
  "minecraft:raw_copper": "minecraft:copper_ingot",
  "minecraft:raw_iron": "minecraft:iron_ingot",
  "minecraft:raw_gold": "minecraft:gold_ingot",
  "minecraft:ancient_debris": "minecraft:netherite_scrap"
};

async function addPickaxeLore(player) {
  try {
    const inventory = player.getComponent('inventory').container;
    
    for (let i = 0; i < inventory.size; i++) {
      const item = inventory.getItem(i);
      if (!item) continue;
      
      if (await isPickaxe(item)) {
        const lore = item.getLore();
        if (!lore.includes(loreLine)) {
          lore.push(loreLine);
          item.setLore(lore);
          inventory.setItem(i, item);
        }
      }
    }
  } catch (e) {
    console.error("Lore adding error:", e);
  }
}

system.runInterval(async () => {
  try {
    for (const player of world.getPlayers()) {
      await addPickaxeLore(player);
    }
  } catch (e) {
    console.error("Interval error:", e);
  }
}, 5);

world.afterEvents.playerBreakBlock.subscribe(async (event) => {
  try {
    const { block, brokenBlockPermutation, player } = event;
    const brokenBlockType = brokenBlockPermutation.type.id;
    
    if (!smeltableOres.includes(brokenBlockType)) return;
    
    const enabled = player.getDynamicProperty("sasEnabled");
    if (!enabled) return;
    
    const distance = player.getDynamicProperty("sasDistance");
    if (typeof distance !== "number") return;
    
    const inventory = player.getComponent('inventory')?.container;
    if (!inventory) return;
    
    const heldItem = inventory.getItem(player.selectedSlotIndex);
    if (!heldItem) return;
    
    const isPick = await isPickaxe(heldItem);
    if (!isPick) return;
    
    const playerLoc = player.location;
    const blockLoc = block.location;
    const dx = playerLoc.x - blockLoc.x;
    const dy = playerLoc.y - blockLoc.y;
    const dz = playerLoc.z - blockLoc.z;
    const actualDistance = Math.sqrt(dx * dx + dy * dy + dz * dz);
    
    if (actualDistance > distance + 1) return;
    
    const nearbyItems = block.dimension.getEntities({
      type: "minecraft:item",
      location: blockLoc,
      maxDistance: distance
    });
    
    for (const item of nearbyItems) {
      const itemComp = item.getComponent("item");
      if (!itemComp) continue;
      
      const typeId = itemComp.itemStack.typeId;
      const smeltedId = smeltingBlocks[typeId];
      if (!smeltedId) continue;
      
      await spawnItem(smeltedId, item);
      await spawnParticle("minecraft:campfire_smoke_particle", item);
      await spawnEntity("minecraft:xp_orb", item);
      await playSound("random.fizz", item);
      item.remove();
    }
    
  } catch (e) {
    console.error("[Simple Autosmelt] Error:", e);
  }
});


world.afterEvents.playerSpawn.subscribe(async (eventData) => {
  try {
    const { player, initialSpawn } = eventData;
    
    if (!initialSpawn) return;

    player.sendMessage("§lSimple §cAuto§eSmelt§r activated");

    let distance = player.getDynamicProperty("sasDistance");
    if (distance === undefined) {
        distance = 16;
        player.setDynamicProperty("sasDistance", distance);
    }
    
    let enabled = player.getDynamicProperty("sasEnabled");
    if (enabled === undefined) {
        enabled = true;
        player.setDynamicProperty("sasEnabled", enabled);
    }
    
    await addPickaxeLore(player);

  } catch (e) {
    console.error("Spawn event error:", e);
  }
});

world.beforeEvents.worldInitialize.subscribe((initEvent) => {
  try {
    
    initEvent.itemComponentRegistry.registerCustomComponent('sas:trigger', {
      async onUse(e) {
        const player = e.source;
        if (!player) return;
        let distance = player.getDynamicProperty("sasDistance");
        if (distance === undefined) {
          distance = 16;
          player.setDynamicProperty("sasDistance", distance);
        }

        let enabled = player.getDynamicProperty("sasEnabled");
        if (enabled === undefined) {
          enabled = true;
          player.setDynamicProperty("sasEnabled", enabled);
        }

        const form = new ModalFormData()
          .title("§lSimple §cAuto§eSmelt§r Config")
          .slider("§cAuto§eSmelt§r Range (may affect §aperformance§r)", 1, 32, 1, distance)
          .toggle("Enable AutoSmelt", enabled);

        const response = await form.show(player);
        if (response.canceled) return;

        player.setDynamicProperty("sasDistance", response.formValues[0]);
        player.setDynamicProperty("sasEnabled", response.formValues[1]);
        player.sendMessage("§lSimple §cAuto§eSmelt§r settings have been updated.");
      },
    });
  } catch (error) {
    console.error("Error during world initialization:", error);
  }
});
